#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle  hnd;
    int                rc;
    int                ret;
    int                devIdx;
    char              *devName  = DEFAULT_DEV;
    char               fullDevName[32];
    char              *pgmname  = argv[0];
    TSYNC_ERROR        err      = TSYNC_SUCCESS;
    unsigned int       index;
    int                timeValid;
    int                ppsValid;

    /* If invalid number of arguments... */
    if (argc != 5)
    {
        printf(" Usage: MR_SetValidity <device index> <index> <time valid>"
               " <pps valid>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    ret = snprintf(fullDevName, sizeof(fullDevName), "%s%s%d",
                   DEVICE_ROOT, devName, devIdx);

    if (ret < 0 || ret >= sizeof(fullDevName))
    {
       printf("Error formating device name\n");
       return (1);
    }

    ret = 0;

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    // Build Set Validity message
    index     = atoi(argv[2]);
    timeValid = atoi(argv[3]);
    ppsValid  = atoi(argv[4]);

    printf("\n");
    printf(" MR (%u) Reference Validity:\n", (index));
    printf("  Time: %u\n", (timeValid));
    printf("  1PPS: %u\n", (ppsValid));

    // Send Set validity transaction
    err = TSYNC_MR_setValidity(hnd, index, timeValid, ppsValid);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        printf("%s: error closing <%s> rc: <%d>\n",
               pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
