#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"

#include "spec_ktsal_en.h"

int main (int argc, char *argv[])
{
    TSYNC_BoardHandle  hnd;
    int                rc;
    int                ret     = 0;
    int                devIdx;
    char              *devName = DEFAULT_DEV;
    char               fullDevName[32];
    char              *pgmname = argv[0];
    TSYNC_ERROR        err     = TSYNC_SUCCESS;
    unsigned int       index;
    TSYNC_StrConfObj   conf;


    /* If invalid number of arguments... */
    if (argc != 3)
    {
        printf(" Usage: STR_GetConf <device index> <index>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Get Mfr/Model message */
    index = atoi(argv[2]);

    // Send Get Mfr/Model transaction
    err = TSYNC_STR_getConf(hnd, index, &conf);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

         /*
      * On definition of the environment variable SPEC_KTSAL_EN
      * to true, print out the information in the format used
      * for the equivalent set's command line.
      */
    if ((getenv(SPEC_KTSAL_EN) != NULL) &&
            *getenv(SPEC_KTSAL_EN) == *ENV_TRUE)
    {
        printf("%s %f %f %f %f %f %f %d %d\n",
            conf.serialnb,
            conf.pos.lat* RAD_TO_DEG,
            conf.pos.lon* RAD_TO_DEG,
            conf.pos.alt,
            conf.ee_axis,
            conf.nn_axis,
            conf.uu_axis,
            conf.geoMode,
            conf.sensLevel);
    }

    printf("\n");
    printf(" STR (%d) Configuration:\n", (index));
    printf("  Serial Number: %s\n", conf.serialnb);
    printf("  Latitude: %f\n", conf.pos.lat* RAD_TO_DEG);
    printf("  Longitude: %f\n", conf.pos.lon* RAD_TO_DEG);
    printf("  Altitude: %f\n", conf.pos.alt);
    printf("  EE axis: %f\n", conf.ee_axis);
    printf("  NN axis: %f\n", conf.nn_axis);
    printf("  UU axis: %f\n", conf.uu_axis);
    printf("  Geolocation mode: %d\n", conf.geoMode);
    printf("  Sensitivity level: %d\n", conf.sensLevel);

    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
