/*
 * COPYRIGHT © 2018 OROLIA GROUP AND/OR ITS AFFILIATES.  ALL RIGHTS ARE STRICTLY RESERVED.
 * THIS SOURCE CODE CONTAINS PROPRIETARY AND CONFIDENTIAL INFORMATION AND DATA AND IS THE SOLE PROPERTY OF OROLIA GROUP
 * AND/OR ITS AFFILIATES. THIS SOURCE CODE MUST NOT BE USED, DISSEMINATED, OR DISTRIBUTED EXCEPT FOR THE AGREED PURPOSE.
 * UNAUTHORIZED USE, REPRODUCTION, OR ISSUE TO ANY THIRD PARTY IS NOT PERMITTED WITHOUT THE PRIOR WRITTEN CONSENT OF THE
 * OROLIA GROUP. THIS SOURCE CODE IS TO BE RETURNED TO THE OROLIA GROUP WHEN THE AGREED PURPOSE IS FULFILLED.
 */

#include "tsync.h"

#include <stdio.h>
#include <stdlib.h>

static TSYNC_ERROR readFromTsync(const char* deviceName, uint32_t* ioNum)
{
    TSYNC_BoardHandle pHandle;
    TSYNC_ERROR err;

    err = TSYNC_open(&pHandle, deviceName);
    if (err != TSYNC_SUCCESS)
    {
        printf(" Error: %s.\n", tsync_strerror(err));
        return err;
    }

    err = TSYNC_SW_getSwitchIoNum(pHandle, ioNum);
    if (err != TSYNC_SUCCESS)
    {
        printf(" Error: %s.\n", tsync_strerror(err));
    }

    err = TSYNC_close(pHandle);
    if (err != TSYNC_SUCCESS)
    {
        printf(" Error: %s.\n", tsync_strerror(err));
    }
    return err;
}

int main(int argc, char* argv[])
{
    if (argc != 2)
    {
        printf(" Usage: SW_GetIONum <device>\n");
        return EXIT_FAILURE;
    }

    char deviceName[32] = {0};
    snprintf(deviceName, sizeof(deviceName), "/dev/tsyncpci%s", argv[1]);

    uint32_t ioNum;

    if (readFromTsync(deviceName, &ioNum) != TSYNC_SUCCESS)
    {
        return EXIT_FAILURE;
    }

    printf("Total IO: %u\n", ioNum);
    return EXIT_SUCCESS;
}