#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle  hnd;
    int                rc;
    int                ret     = 0;
    int                devIdx;
    char              *devName = DEFAULT_DEV;
    char               fullDevName[32];
    char              *pgmname = argv[0];
    TSYNC_ERROR        err     = TSYNC_SUCCESS;
    EC_BLACKOUT_MODE   mode;


    /* If invalid number of arguments... */
    if (argc != 3)
    {
        printf(" Usage: %s <device index> <brightness>\n", pgmname);
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Set Mode message */
    uint32_t val = atoi(argv[2]);
    if (val >= EC_BLACKOUT_NUM || val < EC_BLACKOUT_OFF)
    {
        printf("Wrong argument.\n");
        return 1;
    }

    mode = (EC_BLACKOUT_MODE)val;


    printf("\n");
    printf(" LED blackout mode: %s\n", (mode == EC_BLACKOUT_ON ? "On" : "Off"));

    /* Set LED Mode */
    err = TSYNC_EC_setBlackoutMode(hnd, mode);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
